/*
 * file:       Stepper.c
 * purpose:    functions to handle unipolar steppers (5wire - interface)
 * author:     Santiago
 * copyright:  2008 (c) Creative commons - by-nc-sa - some rights reserved
 *             see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "Config.h"
#include "Base.h"
#include "Stepper.h"


/*
 * timer 0 is stepper speed control
 *         pwm-control is handled by timer 1 (see Base.c)
 */
void stepInit() {
   TCCR0A = 1 << WGM01 | 1 << WGM00;                       // using fast PWM mode with CTC
   TCCR0B = 1 << WGM02                                     // ensures synchronized update of top level 
          | 1 << CS02 | 1 << CS00;                         // prescale: 1024
   OCR0A  = 0xFF;                                          // start with slowest speed
   TIMSK0 = 1 << TOIE0;                                    // enable overflow interrupt

   stepControl.direction = 0;                              // don't move
}


void stepSwitchOn() {
   switch (stepControl.step) {
      case 0:
         PORT_STEP_L1 |= 1 << PIN_STEP_L1; 
         PORT_STEP_L2 &= (uint8_t) ~(1 << PIN_STEP_L2);
         PORT_STEP_L3 &= (uint8_t) ~(1 << PIN_STEP_L3);
         PORT_STEP_L4 |= 1 << PIN_STEP_L4; 
         break;

      case 1:
         PORT_STEP_L1 &= (uint8_t) ~(1 << PIN_STEP_L1);
         PORT_STEP_L2 |= 1 << PIN_STEP_L2; 
         PORT_STEP_L3 |= 1 << PIN_STEP_L3; 
         PORT_STEP_L4 &= (uint8_t) ~(1 << PIN_STEP_L4);
         break;

      case 2:
         PORT_STEP_L1 |= 1 << PIN_STEP_L1; 
         PORT_STEP_L2 |= 1 << PIN_STEP_L2; 
         PORT_STEP_L3 &= (uint8_t) ~(1 << PIN_STEP_L3);
         PORT_STEP_L4 &= (uint8_t) ~(1 << PIN_STEP_L4);
         break;

      case 3:
         PORT_STEP_L1 &= (uint8_t) ~(1 << PIN_STEP_L1);
         PORT_STEP_L2 &= (uint8_t) ~(1 << PIN_STEP_L2);
         PORT_STEP_L3 |= 1 << PIN_STEP_L3; 
         PORT_STEP_L4 |= 1 << PIN_STEP_L4; 
         break;
   }
}


void stepSwitchOff() {
   PORT_STEP_L1 &= (uint8_t) ~(1 << PIN_STEP_L1);
   PORT_STEP_L2 &= (uint8_t) ~(1 << PIN_STEP_L2);
   PORT_STEP_L3 &= (uint8_t) ~(1 << PIN_STEP_L3);
   PORT_STEP_L4 &= (uint8_t) ~(1 << PIN_STEP_L4);
}


/*
 * stepper control / motor direction logic
 */
ISR(TIMER0_OVF_vect) {
   stepControl.steps += stepControl.direction;
   stepControl.step  += stepControl.direction;
   if (stepControl.step > 3)
      stepControl.step = 0;

   if (stepControl.step < 0)
      stepControl.step = 3;
}
